#version 460

uniform ivec3 light_offset;
uniform vec3 sun_light;

in uint data;

out VS_OUT {
	uint voxelTexture;
	uint face;
	vec3 lighting;
	vec3 offset;
} vs_out;

layout(r11f_g11f_b10f) uniform image3D lighting_map;

buffer RenderData {
	int[2048] chunk_x;
	int[2048] chunk_y;
	int[2048] chunk_z;
};

const float face_lighting[10] = float[10](
	0.8,
	0.9,
	0.6,
	0.8,
	0.4,
	0.6,
	0.7,
	0.7,
	0.7,
	0.7
);

const ivec3 face_light_offset[10] = ivec3[10](
	ivec3(1, 0, 0),
	ivec3(0, 1, 0),
	ivec3(0, 0, 1),
	ivec3(-1, 0, 0),
	ivec3(0, -1, 0),
	ivec3(0, 0, -1),
	ivec3(0, 0, 0),
	ivec3(0, 0, 0),
	ivec3(0, 0, 0),
	ivec3(0, 0, 0)
);

void main()
{
	// Unpack the data we retrieved
	const uint x = (data) & 31; // 0-31 x position
	const uint y = (data >> 5) & 31; // 0-31 y position
	const uint z = (data >> 10) & 31; // 0-31 z position

	const uint face = (data >> 15) & 15; // 0-9 (0-15) face

	vs_out.voxelTexture = (data >> 19); // 13 bits, texture index

	// Calculate real positions
	const ivec3 offset = ivec3(chunk_x[gl_DrawID], chunk_y[gl_DrawID], chunk_z[gl_DrawID]);
	vs_out.offset = offset;
	ivec3 light_position = -(light_offset - offset) + ivec3(x, y, z) + face_light_offset[face];

	vs_out.face = face;

	if (any(lessThan(light_position, ivec3(0))) || any(greaterThanEqual(light_position, imageSize(lighting_map).xyz))) {
		vs_out.lighting = sun_light;
	} else {
		vs_out.lighting = imageLoad(lighting_map, light_position).rgb;
	}
	
	vs_out.lighting *= face_lighting[face];

	// Output our position
	gl_Position = vec4(x, y, z, 1.0);
}

